#!/bin/bash
#########################################################################
# vov is a tool that emulates the behavior of a Von-Neumann machine.    #
# Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         #
#                                                                       #
# This file is part of vov.                                             #
#                                                                       #
# vov is free software; you can  redistribute it and/or modify it under #
# the terms of the GNU General  Public License as published by the Free #
# Software Foundation;  either version  2 of the  License, or  (at your #
# option) any later version.                                            #
#                                                                       #
# vov is  distributed in the hope  that it will be  useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or #
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License #
# for more details.                                                     #
#                                                                       #
# You should  have received a  copy of  the GNU General  Public License #
# along  with vov;  if  not,  write to  the  Free Software  Foundation, #
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    #
#########################################################################

set -o errexit
set -o pipefail
set -o nounset
set -o noclobber

# set locale to a reasonable values
export LANG='C'       # you can change this ...
export LC_COLLATE='C' # ... but don't touch this

# a simple signal handler
function _parachute() {
  local -i rc="$?"

  if test "${rc}" -ne 0; then
    echo "[-] ERROR: command died unexpectly. Try to run it manually." >&2
  fi

  exit "${rc}"
}

# handle the EXIT signal
trap '_parachute' EXIT

# generate files
for cmdline in								\
  'shtoolize -q echo move'						\
  'aclocal -I m4'							\
  'autoconf'								\
  'autoheader'								\
  'automake --gnu --add-missing --copy'					\
  './configure --cache-file=/dev/null --enable-maintainer-mode'		\
  'make distcheck' 
  do
    proggy="${cmdline%% *}"

    if test x"${proggy}" = x'make' -a "${#}" -lt 1; then
      break
    fi

    if test x"$(which "$proggy" 2> /dev/null)" = x; then
      echo "[-] ERROR: program \`$proggy' not found on your system." >&2
      exit 111
    fi

    printf "   - running %-10s (cmdline: %s) ...\n" "${proggy##*/}"  "$cmdline"
    $cmdline > /dev/null 2>&1
  done

# silently remove this directory
rm -rf autom4te.cache

exit 0
