/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

/*****************************************************************\
*                           CREDITS                               *
*                                                                 *
* This code  is based on  libowfat (http://www.fefe.de/libowfat/) *
* written by Felix von Leitner and released under the GNU General *
* Public License published by the Free Software Foundation.       *
* In short, libowfat reimplements libdjb under GPL.               *
*                                                                 *
* Since I needed to change some little things and I don't want to *
* add any extra no  standard dependence to vov, I wrote this code *
\*****************************************************************/

#ifndef _BUFFER_H
#define _BUFFER_H


/*
 * a call was interrupted by a signal before any data was written 
 */
#ifndef EINTR
# define EINTR                  (-1)
#endif


/*
 * type of a callback 
 */
typedef int     (*cb_t) (int, const void *, size_t);


/*
 * buffer for the buffered operations 
 */
struct buffer {
    char           *x;		/* actual buffer space */
    unsigned long int p;	/* current position */
    unsigned long int n;	/* current size of string in buffer */
    unsigned long int a;	/* allocated buffer size */
    int             fd;		/* file descriptor were doing op to */
    cb_t            op;		/* operation on the buffer (tipically
				 * read/write) */
};

typedef struct buffer buffer_t;


#ifdef __cplusplus
extern          "C" {
#endif				/* __cplusplus */

    int             get_stdin_line(line_t *);
    int             buffer_putm_flush(buffer_t *, ...);

#ifdef __cplusplus
}
#endif				/* __cplusplus */
#endif				/* _BUFFER_H */
