/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

#ifndef _CPU_H
#define _CPU_H

/*
 * number of symbols in the symbols table 
 */
#ifdef NUM_OF_SYMBOLS
# undef NUM_OF_SYMBOLS
#endif

#define NUM_OF_SYMBOLS			(9)


/*
 * index of the current symbol 
 */
#ifdef SYMTABLE_INDEX
# undef SYMTABLE_INDEX
#endif

#define SYMTABLE_INDEX(x)		((x) / (RAM_NWORDS))


/*
 * argument part of the given opcode 
 */
#ifdef INSTR_ARGUMENT
# undef INSTR_ARGUMENT
#endif

#define INSTR_ARGUMENT(x)		((x) % (RAM_NWORDS))


/*
 * is it an illegal instruction ? 
 */
#ifdef IS_ILLEGAL_SYM
# undef IS_ILLEGAL_SYM
#endif

#define IS_ILLEGAL_SYM(x)		((((x) < 0) || ((x) > ((NUM_OF_SYMBOLS) - 1))) ? 1 : 0)


/*
 * machine cpu 
 */
struct cpu {
    uint16_t        pc;		/* program counter: it holds the address
				 * of the next instruction that will be
				 * executed */
    int32_t         ir;		/* instruction register: it holds the
				 * current instruction */
    int32_t         acc;	/* accumulator: data register used in
				 * arithmetic and logical operations */
};

typedef struct cpu cpu_t;


/*
 * vov symbol 
 */
typedef void    (*symbol_t) (void);


#ifdef __cplusplus
extern          "C" {
#endif				/* __cplusplus */

    void            cpu_run(void);

#ifdef __cplusplus
}
#endif				/* __cplusplus */
#endif				/* _CPU_H */
