/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

#ifndef _HUMAN_OUT_H
#define _HUMAN_OUT_H

/*
 * error prefix 
 */
#ifdef ERROR
# undef ERROR
#endif

#define ERROR                   "[-] ERROR: "


/*
 * warning prefix 
 */
#ifdef WARN
# undef WARN
#endif

#define WARN                    "\n[!] WARNING: "


/*
 * fatal prefix 
 */
#ifdef FATAL
# undef FATAL
#endif

#define FATAL                  "[*] FATAL: "

/*****************************************************************\
 * WARNING: since  ISO C does not  permit named  variadic macros, *
 * even if gcc does I don't want to use things like:              *
 *                                                                *
 * #define debug(fmt,args...)	printf(fmt,args);                 *
 *                                                                *
 * because it  isn't standard.  In C99 anonymous  variadic macros *
 * were introduced  (this set of  tokens replaces  the identifier *
 * `__VA_ARGS__' in the macro body wherever it appears) therefore *
 * I use them.                                                    *
 * Please note that in standard C, you are  not allowed  to leave *
 * the variable argument out entirely; you are allowed to pass an *
 * empty argument. For example, this invocation is invalid in ISO *
 * C, because there is no comma after the string:                 *
 *                                                                *
 *		debug("hello\n");                                 *
 *                                                                *
 * To solve this problem, GNU CPP behaves  specially for variable *
 * arguments used with the token paste operator, `##'. If instead *
 * you write:                                                     *
 *                                                                *
 * #define debug(fmt,...)	printf(fmt, ## __VA_ARGS__);      *
 *                                                                *
 * and if  the variable arguments  are omitted or empty, the `##' *
 * operator causes the preprocessor to remove the comma before it.*
 * If  you  do  provide  some  variable  arguments  in your macro *
 * invocation GNU CPP does not complain about the paste operation * 
 * and instead places the variable arguments after the comma.     *
 *                                                                *
 * Again, since it is a GCC-ism  I don't want to use it;  this do *
 * not cause problems in vov, because calling these macro without *
 * arguments makes no sense.                                      *
 \****************************************************************/

/*
 * free all memory, print a message on stderr and then die 
 * to sys with the specified return code
 */
#define die(rc,...) \
do { \
  extern buffer_t *buffer_2; \
\
  (void)buffer_putm_flush(buffer_2, FATAL, __VA_ARGS__, NULL); \
  free_exit(rc); \
} \
while(0)


/*
 * print a message on stdout: if an error occurred while printing
 * the message then free all memory and die to sys
 */
#define msg(...) \
do { \
  extern buffer_t *buffer_1; \
\
  if (-1 == buffer_putm_flush(buffer_1, __VA_ARGS__, NULL)) \
    free_exit(69); \
} \
while(0)


/*
 * print a message on sterr: if an error occurred while printing
 * the message then free all memory and die to sys
 */
#define wrn(...) \
do { \
  extern buffer_t *buffer_2; \
\
  if (-1 == buffer_putm_flush(buffer_2, WARN, __VA_ARGS__, NULL)) \
    free_exit(69); \
} \
while(0)


/*
 * print a message on sterr: if an error occurred while printing
 * the message then free all memory and die to sys
 */
#define err(...) \
do { \
  extern buffer_t *buffer_2; \
\
  if (-1 == buffer_putm_flush(buffer_2, ERROR, __VA_ARGS__, NULL)) \
    free_exit(69); \
} \
while(0)



#ifdef __cplusplus
extern          "C" {
#endif				/* __cplusplus */

    void            free_exit(int);

#ifdef __cplusplus
}
#endif				/* __cplusplus */
#endif				/* _HUMAN_OUT_H */
