/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

#ifndef _RAM_H
#define _RAM_H

/*
 * height of current tree node 
 */
#ifdef GET_HEIGHT
# undef GET_HEIGHT
#endif

#define GET_HEIGHT(x)		((NULL == (x)) ? (-1) : ((x)->height))


/*
 * max height 
 */
#ifdef MAX_HEIGHT
# undef MAX_HEIGHT
#endif

#define MAX_HEIGHT(x,y)   	(((x) > (y)) ? (x) : (y))


/*
 * max number of ram words 
 */
#ifdef RAM_NWORDS
# undef RAM_NWORDS
#endif

#define RAM_NWORDS		(1000)


/*
 * some debug macro 
 */
#ifdef DEBUG
# ifdef DEBUG_MKSPACES
#  undef DEBUG_MKSPACES
# endif

# define DEBUG_MKSPACES(x) \
do { \
  size_t i; \
\
  for (i = 0; i < sizeof((x)) - 1; i++) \
    (x)[i] = ' '; \
\
  (x)[i] = '\0'; \
} \
while(0)
#endif				/* ! DEBUG */


/*
 * single word 
 */
struct word {
    uint16_t        offset;
    int32_t         opcode;
    struct word    *left;
    struct word    *right;
    int             height;
};

typedef struct word ram_t;


#ifdef __cplusplus
extern          "C" {
#endif				/* __cplusplus */

    int             add_word(uint16_t, int32_t, ram_t **);
    ram_t          *word_seek(uint16_t, ram_t *);
    void            destroy_ram(ram_t *);
#ifdef DEBUG
    void            print_tree_ram(ram_t *);
    void            print_flat_ram(ram_t *);
#endif

#ifdef __cplusplus
}
#endif				/* __cplusplus */
#endif				/* _RAM_H */
