dnl @synopsis AC_C_ALWAYS_INLINE([ACTIONS-IF-FOUND] [, ACTIONS-IF-NOT-FOUND])
dnl
dnl: Checks     if     your     compiler     supports     the     attribute
dnl: `__always_inline__'. This  macro  require   AC_C_INLINE  (coming  from
dnl: autotools  distribution)  because  it  sets the  keyword  `inline'  to
dnl: something  appropriate. First  of  all   this  macro  checks  if  your
dnl: compiler supports the  keyword `inline' then checks  for the attribute
dnl: `__always_inline__'. If all tests  succeded then defines ALWAYS_INLINE
dnl: preprocessor  directive to  `__attribute__ ((__always_inline__))'  and
dnl: runs any user defined action (1st parameter). If no attribute is found
dnl: or if your  compiler don't support the `inline'  keyword, then defines
dnl: ALWAYS_INLINE  to   empty  and  runs  any  user  defined  action  (2nd
dnl: parameter).
dnl
dnl Examples:
dnl    AC_C_ALWAYS_INLINE([AM_CONDITIONAL(BAR, true)],[AM_CONDITIONAL(BAR, false)])
dnl
dnl    AC_C_ALWAYS_INLINE([AM_CONDITIONAL(FOOBAR, true)])
dnl
dnl    AC_C_ALWAYS_INLINE
dnl
dnl @category C
dnl @author Davide Scola <davide.scola@gmail.com>
dnl @version 2005-02-08
dnl @license AllPermissive


AC_DEFUN([AC_C_ALWAYS_INLINE],
[
AH_TEMPLATE([ALWAYS_INLINE],[Define to something appropriate if your compiler support it.])

AC_REQUIRE([AC_C_INLINE])
AC_MSG_CHECKING([for __attribute__ ((__always_inline__))])

if test x"$ac_cv_c_inline" = xinline; then
  ac_cv_always_inline_cflags_backup="$CFLAGS"
  CFLAGS="$CFLAGS -Wall -Werror"
  ac_cv_always_inline=no  
  
  AC_COMPILE_IFELSE(
  [AC_LANG_PROGRAM( 
  [[
  ]],
  [[
static inline __attribute__ ((__always_inline__)) void func (void);
  ]], 
  )], [ac_cv_always_inline=yes], [ac_cv_always_inline=no])
  
  CFLAGS="$ac_cv_always_inline_cflags_backup"
  AC_MSG_RESULT($ac_cv_always_inline)
  
  if test x$ac_cv_always_inline = xyes; then
    # define this macro
    AC_DEFINE_UNQUOTED([ALWAYS_INLINE],[__attribute__ ((__always_inline__))])

    # run user defined if-found-actions
    ifelse([$1], , :, [$1])
  else
    # define an empty attribute
    AC_DEFINE_UNQUOTED([ALWAYS_INLINE],[])

    # run user defined not-found-actions
    ifelse([$2], , :, [$2])
  fi
else
  AC_MSG_RESULT(no)

  # define an empty attribute
  AC_DEFINE_UNQUOTED([ALWAYS_INLINE],[])

  # no always inline attribute found on this system; run not-found-actions
  ifelse([$2], , :, [$2])
fi
])

