dnl @synopsis AC_CHANGE_LIMIT([ACTIONS-IF-FOUND] [, ACTIONS-IF-NOT-FOUND])
dnl
dnl Checks if you  can call [gs]etrlimit functions  in your program. First
dnl of all this  macro tries to compile a simple  source that would change
dnl the  limit of  RLIMIT_DATA using  [gs]etrlimit functions. Please  note
dnl this is necessary  because, on some  environements (i.e. cross mingw),
dnl checking only the functions isn't correct (they seems to exist but the
dnl compilation will fail because `struct rlimit' isn't know). If the test
dnl succeded then  defines HAVE_CHANGE_LIMIT  preprocessor directive  to 1
dnl and runs any  user defined action (1st parameter),  otherwise runs the
dnl if-not-found actions (2nd parameter).
dnl
dnl Examples:
dnl    AC_CHANGE_LIMIT([AM_CONDITIONAL(BAR, true)],[AM_CONDITIONAL(BAR, false)])
dnl
dnl    AC_CHANGE_LIMIT([AM_CONDITIONAL(FOOBAR, true)])
dnl
dnl    AC_CHANGE_LIMIT
dnl
dnl @category C
dnl @author Davide Scola <davide.scola@gmail.com>
dnl @version 2005-02-08
dnl @license AllPermissive


AC_DEFUN([AC_CHANGE_LIMIT],
[
AH_TEMPLATE([HAVE_CHANGE_LIMIT],[Define to 1 if you can change memory limits.])

AC_MSG_CHECKING([whether the system supports memory limit])

ac_cv_change_limit=no

AC_COMPILE_IFELSE(
[AC_LANG_PROGRAM(
[[
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

#ifdef __DJGPP__
# error choke me
#endif
]],
[[
struct rlimit z;

(void)getrlimit(RLIMIT_DATA, &z);
z.rlim_cur = z.rlim_max;
(void)setrlimit(RLIMIT_DATA, &z)
]],
)], [ac_cv_change_limit=yes], [ac_cv_change_limit=no])

AC_MSG_RESULT($ac_cv_change_limit)

if test x$ac_cv_change_limit = xyes; then
  # define this macro
  AC_DEFINE_UNQUOTED([HAVE_CHANGE_LIMIT],[1])

  # run user defined if-found-actions
  ifelse([$1], , :, [$1])
else
  # run user defined not-found-actions
  ifelse([$2], , :, [$2])
fi
])

