dnl @synopsis AC_FUNC_SIGACTION([ACTIONS-IF-FOUND] [, ACTIONS-IF-NOT-FOUND])
dnl
dnl Checks  if  you   can  call  sigaction's  family   functions  in  your
dnl program. This  macro  require  AC_TYPE_SIGNAL (coming  from  autotools
dnl distribution)  because  it  sets  the signal-handler  return  type  to
dnl something  appropriate. First of  all this  macro tries  to compile  a
dnl simple  source  that would  sets  a  signal-handler for  SIGINT  using
dnl sigaction's family  functions. Please note this is  necessary because,
dnl on some environements (i.e.  cross mingw), checking only the functions
dnl isn't correct (they seems to exist  but the compilation  will fail due
dnl `struct  sigaction' isn't  know). If  the test  succeded then  defines
dnl HAVE_SIGACTION preprocessor directive  to 1 and runs  any user defined
dnl action (1st  parameter), otherwise runs the  if-not-found actions (2nd
dnl parameter).
dnl
dnl Examples:
dnl    AC_FUNC_SIGACTION([AM_CONDITIONAL(BAR, true)],[AM_CONDITIONAL(BAR, false)])
dnl
dnl    AC_FUNC_SIGACTION([AM_CONDITIONAL(FOOBAR, true)])
dnl
dnl    AC_FUNC_SIGACTION
dnl
dnl @category C
dnl @author Davide Scola <davide.scola@gmail.com>
dnl @version 2005-02-08
dnl @license AllPermissive


AC_DEFUN([AC_FUNC_SIGACTION],
[
AH_TEMPLATE([HAVE_SIGACTION],[Define to 1 if you have the `sigaction' function.])

AC_REQUIRE([AC_TYPE_SIGNAL])
AC_MSG_CHECKING([for sigaction])

ac_cv_have_sigaction=no

AC_COMPILE_IFELSE(
[AC_LANG_PROGRAM(
[[
#include <signal.h>
RETSIGTYPE func(int z) { z = z; }
]],
[[
struct sigaction x;

x.sa_handler = func;
x.sa_flags = 0;
(void)sigemptyset(&x.sa_mask);
(void)sigaction(SIGINT, &x, (struct sigaction *)0);
]],
)], [ac_cv_have_sigaction=yes], [ac_cv_have_sigaction=no])

AC_MSG_RESULT($ac_cv_have_sigaction)

if test x$ac_cv_have_sigaction = xyes; then
  # define this macro
  AC_DEFINE_UNQUOTED([HAVE_SIGACTION],[1])

  # run user defined if-found-actions
  ifelse([$1], , :, [$1])
else
  # run user defined not-found-actions
  ifelse([$2], , :, [$2])
fi
])

