/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef __MINGW32__
# include <stdio.h>             /* STD(IN|OUT|ERR)_FILENO and NULL here */
#endif

#include <unistd.h>

#ifdef HAVE_STDINT_H
# include <stdint.h>
#endif

#include "line.h"
#include "buffer.h"
#include "ram.h"
#include "cpu.h"
#include "vov.h"
#include "scanner.h"
#include "bootstrap.h"
#include "human_out.h"
#include "options.h"


/*
 * extern variables 
 */
extern vov_t    vov;
extern uint8_t  options;
extern unsigned long nstatement;


/*
 * very small and simple bootstrap program: it reads opcodes from stdin
 * and store them on the machine's RAM until 8000 is given, then it starts 
 * the program. 
 */
static inline ALWAYS_INLINE int
create_bios(void)
{
    register uint16_t offset;
    int32_t         rom[ROM_NCODES] = {
	2000,			/* read new opcode form stdin */
	4000,			/* "dynamic" instruction used to store the 
				 * given opcode on j (j starts from 0) */
	1999,			/* subtract the content of the word 999 */
	7000,			/* if it's 0 (therefore the user inserted
				 * 8000), go to offset 0 (start user
				 * program) */
	5988,			/* load the "dynamic" instruction */
	997,			/* add 1 to the instruction */
	4988,			/* rewrite the updated instruction */
	1998,			/* check if we'll overwrite this program
				 * during the next loop (substract the
				 * instruction limit) */
	7999,			/* if it's 0, terminate the boot instead
				 * of having an undefined behavior */
	6987,			/* let's read the next opcode */
	1,			/* constant 1; used to increment the
				 * "dynamic" instruction */
	4987,			/* STORE instruction limit */
	8000			/* used to jump to the user program or to
				 * terminate this program on error */
    };

    /*
     * put this code on the machine RAM 
     */
    for (offset = 0; offset < ROM_NCODES; offset++) {
	if (!add_word((offset + ROM_OFFSET), rom[offset], &vov.ram))
	    return 0;
    }

    /*
     * set program counter 
     */
    vov.cpu.pc = ROM_OFFSET;
    return 1;
}


/*
 * if a filename is given (or if it isn't set and the user don't specified
 * to go in interactive mode) then this function will read the code from it
 * (using stdin if the file name is NULL). Otherwise let's boot-up using the
 * bootstrap code.
 */
void
bootstrap(const char *fname)
{
    int             ret;

    if ((NULL == fname) && OPTION_TEST(IACTIVE)) {
	if (!OPTION_TEST(BEQUIET))
	    version(DONT_EXIT);

	ret = create_bios();
    }

    else {
	if ((!OPTION_TEST(BEQUIET)) && (NULL == fname))
	    version(DONT_EXIT);

	ret = load_external_program(fname);

	if ((1 == ret) && (0 == nstatement))
	    die(111, "there is nothing to run here\n");
    }

    /*
     * if something went wrong, free all memory and exit immediately 
     */
    if (0 == ret)
	die(111, "cannot boot the machine\n");
}
