/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef __MINGW32__
# include <stdio.h>             /* STD(IN|OUT|ERR)_FILENO and NULL here */
#endif

#include <unistd.h>

#ifdef HAVE_STDINT_H
# include <stdint.h>
#endif

#include "line.h"
#include "buffer.h"
#include "ram.h"
#include "cpu.h"
#include "vov.h"
#include "human_out.h"
#include "utils.h"
#include "sym_table.h"
#include "options.h"


/*
 * extern variables 
 */
extern vov_t    vov;
extern uint8_t  options;
extern int      exitasap;

/*
 * global variables 
 */
/*
 * - SOMEBODY SET UP US THE BOMB. WE GET SIGNAL(c) 
 */
int             signum;


/*
 * symbols table 
 */
static const symbol_t cpu_symbols[NUM_OF_SYMBOLS] = {
    vov_add, vov_sub, vov_read, vov_write,
    vov_store, vov_load, vov_jmp, vov_jzero,
    vov_halt
};


/*
 * fetch current instruction 
 */
static inline ALWAYS_INLINE void
fetch_instruction(void)
{
    ram_t          *word;
    char            stra[FMT_ULONG];

    if (OPTION_TEST(VERBOSE)) {
	stra[stringify(stra, vov.cpu.pc)] = '\0';
	msg("\n* fetching offset  ", stra, "\n");
    }

    if (NULL == (word = word_seek(vov.cpu.pc, vov.ram))) {
	stra[stringify(stra, vov.cpu.pc)] = '\0';
	die(111,
	    "SIGSEGV received: Segmentation fault while fetching offset ",
	    stra, "\n");
    }

    /*
     * set registers 
     */
    vov.cpu.ir = word->opcode;
    vov.cpu.pc += 1;

    if (OPTION_TEST(VERBOSE)) {
	stra[stringify(stra, vov.cpu.ir)] = '\0';
	msg("\t- instruction register set to ", stra, "\n");
    }
}


/*
 * decode current instruction 
 */
static inline ALWAYS_INLINE void
decode_instruction(int32_t * i)
{
    char            stra[FMT_ULONG];

    *i = SYMTABLE_INDEX(vov.cpu.ir);

    if (OPTION_TEST(VERBOSE)) {
	stra[stringify(stra, *i)] = '\0';
	msg("\t- trying to decode instruction ", stra, "\n");
    }

    if (IS_ILLEGAL_SYM(*i)) {
	stra[stringify(stra, *i)] = '\0';
	die(111, "SIGILL received: Illegal instruction ", stra, "\n");
    }

    if (OPTION_TEST(VERBOSE)) {
	stra[stringify(stra, *i)] = '\0';
	msg("\t- exec instruction ", stra, "\n");
    }
}


/*
 * cpu decoder 
 */
void
cpu_run(void)
{
    int             i;
    char            stra[FMT_ULONG];

    while (1 != exitasap) {
	/*
	 * 1st step: fetch current instruction 
	 */
	fetch_instruction();

	/*
	 * 2nd step: decode current instruction 
	 */
	decode_instruction(&i);

	/*
	 * 3rd step: execute current instruction 
	 */
	cpu_symbols[i] ();
    }

    if (0 != signum) {
	/*
	 * reset this flag or nothing will be printed 
	 */
	exitasap = 0;

	/*
	 * print message 
	 */
	stra[stringify(stra, signum)] = '\0';
	wrn("caught signal ", stra, ": exiting now!\n");
    }
}
