/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef __MINGW32__
# include <stdio.h>             /* STD(IN|OUT|ERR)_FILENO and NULL here */
#endif

#include <stdlib.h>

#if ! HAVE_REALLOC
# include "rpl_realloc.h"
#endif

#include "line.h"

/*
 * check if current line has enough memory to handle len bytes,
 * otherwise realloc it
 */
static inline ALWAYS_INLINE int
grows_line_size(line_t * ln, unsigned long int len)
{
    register char  *tmp;
    register int    wanted = len + (len >> 3) + 30;	/* heuristic from
							 * djb ... I trust 
							 * him :P */

    if ((NULL == ln->s) || (ln->a < len)) {
	if (NULL == (tmp = realloc(ln->s, wanted)))
	    return 0;

	ln->a = wanted;
	ln->s = tmp;
    }

    return 1;
}


/*
 * check if line already has some memory allocated; if so looking for
 * integer overflow and return 0 if any, otherwise call grows_line_size
 */
int
safe_grows_line(line_t * ln, unsigned long size)
{
    if (NULL != ln->s) {
	if (ln->len + size < size)
	    return 0;

	return grows_line_size(ln, ln->len + size);
    }

    return grows_line_size(ln, size);
}


/*
 * add one byte to given line and copy ch (return 0 on error)
 */
int
line_tail(line_t * ln, const char *ch)
{
    if (safe_grows_line(ln, 1)) {
	ln->s[ln->len] = *ch;
	++ln->len;

	return 1;
    }

    return 0;
}
