/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef __MINGW32__
# include <stdio.h>             /* STD(IN|OUT|ERR)_FILENO and NULL here */
# include <stdlib.h>		/* _exit here */
#endif

#include <unistd.h>

#ifndef __DJGPP__
# include <getopt.h>
#endif

#ifdef HAVE_STDINT_H
# include <stdint.h>
#endif

#include "line.h"
#include "buffer.h"
#include "human_out.h"
#include "utils.h"
#include "options.h"


/*
 * global variables 
 */
uint8_t         options;
extern int      optind;
extern char    *optarg;


/*
 * show version 
 */
void
version(int action)
{
    register int    i = 0;
    const char     *versiondata[] = {
	PACKAGE_STRING "\n",
	"Copyright 2001/08 Davide Scola <" PACKAGE_BUGREPORT ">\n",
	"vov is free software, covered by the GNU General Public License, and you\n",
	"are welcome  to change it and/or  distribute copies of it  under certain\n",
	"conditions. See the source for copying conditions.\n",
	"There is  NO warranty;  not even  for MERCHANTABILITY  or FITNESS  FOR A\n",
	"PARTICULAR PURPOSE.\n",
	"This vov was configured as \"" VOV_TARGET "\".\n",
	NULL
    };

    while (versiondata[i] != NULL)
	msg(versiondata[i++]);

    if (DONT_EXIT != action)
	_exit(action);
}


/*
 * show help 
 */
static inline ALWAYS_INLINE void
help(void)
{
    register int    i = 0;
    const char     *helpdata[] = {
	"This is vov.  Usage:\n",
	"\n",
	"    vov [options] [vov script]\n",
	"\n",
	"Options:\n",
	"\n",
	"  -h                 Print this message.\n",
	"  -i                 Run in interactive mode.\n",
	"  -m BYTES           Limit the maximum used memory to BYTES bytes.\n",
	"  -q                 Do not print version number on startup.\n",
	"  -t SECS            Limit the maximum used cpu time to SECS secs.\n",
	"  -v                 Run in verbose mode.\n",
	"  -V                 Print version information and then exit.\n",
	"\n",
	"For more information, consult the vov manual (available in various formats).\n",
	"Report bugs to \"" PACKAGE_BUGREPORT "\".\n",
	NULL
    };

    while (helpdata[i] != NULL)
	msg(helpdata[i++]);

    _exit(0);
}


/*
 * parse command line 
 */
void
parse_cmdline(int *nitems, char ***inames)
{
#ifdef HAVE_CHANGE_LIMIT
    unsigned long int maxmem = DEF_MEM_LMT;
    unsigned long int maxcpu = DEF_CPU_LMT;
#endif
    int             flag;

    while ((flag = getopt(*nitems, *inames, "him:qt:vV")) != -1) {
	switch (flag) {
	case 'h':		/* show help */
	    help();
	    break;

	case 'i':		/* interactive */
	    OPTION_SETS(IACTIVE);
	    break;

	case 'm':		/* memory soft limit */
#ifdef HAVE_CHANGE_LIMIT
	    ulongify(optarg, &maxmem);
#endif
	    break;

	case 'q':		/* be quiet */
	    OPTION_SETS(BEQUIET);
	    break;

	case 't':		/* cpu time soft limit */
#ifdef HAVE_CHANGE_LIMIT
	    ulongify(optarg, &maxcpu);
#endif
	    break;

	case 'v':		/* be verbose */
	    OPTION_SETS(VERBOSE);
	    break;

	case 'V':		/* show version */
	    version(EXIT_NOW0);
	    break;
	}
    }

#ifdef HAVE_CHANGE_LIMIT
    limit_memory(maxmem);
    limit_cpu(maxcpu);
#endif

    *nitems -= optind;
    *inames += optind;
}
