%{
/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef __MINGW32__
# include <stdio.h>		/* STD(IN|OUT|ERR)_FILENO and NULL here */
#endif

#include <unistd.h>

#ifdef HAVE_STDINT_H
# include <stdint.h>
#endif

#include "line.h"
#include "buffer.h"
#include "ram.h"
#include "cpu.h"
#include "vov.h"
#include "human_out.h"
#include "utils.h"

#if ! HAVE_MALLOC
# if HAVE_MALLOC_H
#  include <malloc.h>
# else
#  include "rpl_malloc.h"
# endif
#endif


/* add these prototypes (provided by lexer) to make the compiler happy */
void yyerror(const char *);
int yylex(void);

/* extern variables */
extern unsigned long nline;
extern unsigned long nchars;
extern unsigned long nstatement;
extern vov_t vov;

%}

%token MEMORY
%token NUMBER
%token COMMENT

%%
input : /* empty */
      | input statement
      ;

statement : '\n'
	  | COMMENT
	  | MEMORY '[' number ']' '=' number ';'	{ 
						  	  if (++nstatement > RAM_NWORDS) {
							    err("Your program contains too many statement\n");
							    YYABORT;
							  }

							  if ($3 > (RAM_NWORDS - 1)) {
							    char strnline[FMT_ULONG];
							    char stroffset[FMT_ULONG];

							    strnline[stringify(strnline, nline)] = '\0';
							    stroffset[stringify(stroffset, $3)] = '\0';

							    err("line ", strnline, ": invalid offset ", stroffset, "\n");
							    YYABORT;
							  }

							  if (SYMTABLE_INDEX($6) > (NUM_OF_SYMBOLS - 1)) {
							    char strnline[FMT_ULONG];
							    char stropcode[FMT_ULONG];

							    strnline[stringify(strnline, nline)] = '\0';
							    stropcode[stringify(stropcode, $6)] = '\0';

							    err("line ", strnline, ": invalid opcode ", stropcode, "\n");
							    YYABORT;
							  }

							  if (!add_word($3, $6, &vov.ram))
							    YYABORT;
							}
	  ;

number: NUMBER						{
							  $$ = $1;
							}
      ;
%%
