/***********************************************************************\
* vov is a tool that emulates the behavior of a Von-Neumann machine.    *
* Copyright (C) 2001-2008 Davide Scola <davide.scola@gmail.com>         *
*                                                                       *
* This file is part of vov.                                             *
*                                                                       *
* vov is free software; you can  redistribute it and/or modify it under *
* the terms of the GNU General  Public License as published by the Free *
* Software Foundation;  either version  2 of the  License, or  (at your *
* option) any later version.                                            *
*                                                                       *
* vov is  distributed in the hope  that it will be  useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
* for more details.                                                     *
*                                                                       *
* You should  have received a  copy of  the GNU General  Public License *
* along  with vov;  if  not,  write to  the  Free Software  Foundation, *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    *
\***********************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef __MINGW32__
# include <stdio.h>             /* STD(IN|OUT|ERR)_FILENO and NULL here */
#endif

#include <stdlib.h>
#include <unistd.h>

#ifdef HAVE_STDINT_H
# include <stdint.h>
#endif

#include "line.h"
#include "ram.h"
#include "cpu.h"
#include "vov.h"
#include "bootstrap.h"
#include "parachute.h"
#include "options.h"

#ifdef DEBUG
# include "allocator.h"
#endif


/*
 * global machines
 */
vov_t           vov;
line_t          line;		/* used by buffered read */


#ifdef DEBUG
extern uint8_t  options;
#endif


int
main(int argc, char *argv[])
{
    init_signals();

    parse_cmdline(&argc, &argv);

    /*
     * in debug mode, vov will be verbose 
     */
#ifdef DEBUG
    OPTION_SETS(VERBOSE);
#endif

    /*
     * boot the machine 
     */
    bootstrap(((argc > 0) ? argv[0] : NULL));

    install_parachute();

#ifdef DEBUG
    print_tree_ram(vov.ram);
    print_flat_ram(vov.ram);
    print_memory();
#endif

    /*
     * run the machine 
     */
    cpu_run();

    /*
     * free all memory 
     */
    destroy_ram(vov.ram);
    line_cleanup(line);
    return 0;
}
